# coding=utf8
# Copyright (c) 2020 Arseniy Kuznetsov
##
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
##
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


from mktxp.collector.base_collector import BaseCollector
from mktxp.datasource.container_ds import ContainerDataSource
from mktxp.datasource.package_ds import PackageMetricsDataSource


class ContainerCollector(BaseCollector):
    '''Neighbor Collector'''

    @staticmethod
    def collect(router_entry):
        if router_entry.config_entry.container and PackageMetricsDataSource.is_package_installed(router_entry, package_name='container'):
            metric_labels = ['name', 'repo', 'os', 'arch', 'status']
            records = ContainerDataSource.metric_records(router_entry, metric_labels=metric_labels)
            metrics = BaseCollector.info_collector('container', 'Containers', records, metric_labels=metric_labels)
            yield metrics
